@@ -1,6 +1,6 @@
 /*
 * this file is part of the oxygen gtk engine
-* Copyright (c) 2010 Hugo Pereira Da Costa <hugo.pereira@free.fr>
+* Copyright (c) 2010 Hugo Pereira Da Costa <hugo@oxygen-icons.org>
 *
 * This  library is free  software; you can  redistribute it and/or
 * modify it  under  the terms  of the  GNU Lesser  General  Public
@@ -167,12 +167,15 @@ namespace Oxygen
 
         if( !_target ) return;
 
-        gint xPointer, yPointer;
-        gdk_window_get_pointer( gtk_widget_get_window( _target ), &xPointer, &yPointer, 0L );
-
         GdkWindow* window( gtk_widget_get_window( _target ) );
         GdkWindow* childWindow( 0L );
 
+        // get mouse pointer position
+        gint xPointer(0), yPointer(0);
+        GdkDeviceManager* manager( gdk_display_get_device_manager( gtk_widget_get_display( _target ) ) );
+        GdkDevice* pointer( gdk_device_manager_get_client_pointer( manager ) );
+        gdk_window_get_device_position( window, pointer, &xPointer, &yPointer, 0L);
+
         // reset offset
         int xOffset(0);
         int yOffset(0);
@@ -249,9 +252,9 @@ namespace Oxygen
         if( !topLevel ) return false;
 
         return
-            GTK_WIDGET_VISIBLE( menu ) &&
-            GTK_WIDGET_REALIZED( topLevel ) &&
-            GTK_WIDGET_VISIBLE( topLevel );
+            gtk_widget_get_visible( menu ) &&
+            gtk_widget_get_realized( topLevel ) &&
+            gtk_widget_get_visible( topLevel );
     }
 
     //________________________________________________________________________________
