@@ -20,6 +20,10 @@
 
 #include "oxygenpaneddata.h"
 
+#ifdef GDK_WINDOWING_X11
+#include <gdk/gdkx.h>
+#endif
+
 #include <cassert>
 
 namespace Oxygen
@@ -48,16 +52,20 @@ namespace Oxygen
         if( !GTK_IS_PANED( widget ) ) return;
 
         // load cursor if needed
+        #ifdef GDK_WINDOWING_X11
         if( !_cursorLoaded )
         {
             assert( !_cursor );
-
             GdkDisplay *display( gtk_widget_get_display( widget ) );
-            const bool vertical( gtk_orientable_get_orientation( GTK_ORIENTABLE( widget ) ) == GTK_ORIENTATION_VERTICAL );
-            _cursor = gdk_cursor_new_from_name( display, vertical ? "col-resize":"row-resize" );
-            _cursorLoaded = true;
+            if( GDK_IS_X11_DISPLAY( display ) )
+            {
+                const bool vertical( gtk_orientable_get_orientation( GTK_ORIENTABLE( widget ) ) == GTK_ORIENTATION_VERTICAL );
+                _cursor = gdk_cursor_new_from_name( display, vertical ? "col-resize":"row-resize" );
+                _cursorLoaded = true;
+            }
 
         }
+        #endif
 
         // assign to widget
         if( _cursor )
